import sendResponse from "@backend/sendResponse"
import { rejectionErrors } from "@lib/backend/errors"
import { getPayloadwithChecksum } from "@lib/getPayloadwithChecksum"
import prisma from "@lib/prisma"
import { GamePropsSchema } from "@lib/zodSchemas"
import type { NextApiRequest, NextApiResponse } from "next"
import { getServerSession } from "next-auth"
import { authOptions } from "../auth/[...nextauth]"

export const gameSelects = {
  select: {
    id: true,
    allowChat: true,
    allowMarkDraw: true,
    allowSpecials: true,
    allowSpectators: true,
    state: true,
    gamePin: {
      select: {
        pin: true,
      },
    },
    users: {
      select: {
        id: true,
        index: true,
        chats: {
          select: {
            id: true,
            event: true,
            message: true,
            createdAt: true,
          },
        },
        moves: {
          select: {
            index: true,
            type: true,
            x: true,
            y: true,
            orientation: true,
          },
        },
        ships: {
          select: {
            size: true,
            variant: true,
            x: true,
            y: true,
            orientation: true,
          },
        },
        hits: {
          select: {
            x: true,
            y: true,
            hit: true,
          },
        },
        user: {
          select: {
            id: true,
            name: true,
          },
        },
      },
    },
  },
}

export const getAnyGame = (gameId: string) => {
  const game = prisma.game.findFirst({
    where: {
      NOT: {
        state: "ended",
      },
      id: gameId,
    },
    ...gameSelects,
  })
  return game
}

export const getAnyRunningGame = (userId: string) => {
  const game = prisma.game.findFirst({
    where: {
      NOT: {
        state: "ended",
      },
      users: {
        some: {
          userId,
        },
      },
    },
    ...gameSelects,
  })
  return game
}

export function composeBody(
  gameDB: NonNullable<Awaited<ReturnType<typeof getAnyRunningGame>>>,
): GamePropsSchema {
  const { gamePin, ...game } = gameDB
  const users = gameDB.users
    .map(({ user, ...props }) => ({
      ...props,
      ...user,
    }))
    .sort((user1, user2) => user1.index - user2.index)
  let activeIndex = undefined
  if (game.state === "running") {
    const l1 = game.users[0].moves.length
    const l2 = game.users[1].moves.length
    activeIndex = l1 > l2 ? 1 : 0
  }
  const payload = {
    game: game,
    gamePin: gamePin?.pin ?? null,
    users,
    activeIndex,
  }
  return getPayloadwithChecksum(payload)
}

export default async function running(
  req: NextApiRequest,
  res: NextApiResponse<GamePropsSchema>,
) {
  const session = await getServerSession(req, res, authOptions)

  if (!session?.user) {
    return sendResponse(req, res, rejectionErrors.unauthorized)
  }

  const { email, id } = session.user

  const game = await getAnyRunningGame(id)

  if (!game)
    return sendResponse(req, res, {
      message: `User <${email}> is in no game.`,
      statusCode: 204,
      type: ["debug", "infoCyan"],
    })

  const body = composeBody(game)

  return sendResponse(req, res, {
    message: `User <${email}> asked for game: ${game.id}`,
    statusCode: 200,
    body,
    type: ["debug", "infoCyan"],
  })
}
